@@ -11,7 +11,7 @@ use Plift::Context;
 use Digest::MD5 qw/ md5_hex /;
 use namespace::clean;
 
-our $VERSION = "0.06";
+our $VERSION = "0.07";
 
 use constant {
     XML_DOCUMENT_NODE      => 9,
@@ -43,6 +43,7 @@ sub BUILD {
         Handler::Wrap
         Handler::Render
         Handler::Snippet
+        Handler::Meta
     /;
 
     # plugins
@@ -78,14 +79,14 @@ sub template {
 
     # path copy for the load_template closure
     # this way we do not expose the engine nor the path to the context object
-    my @paths = @{ $options->{paths} || $self->paths };
-    my @ns    = @{ $options->{snippet_namespaces} || $self->snippet_namespaces };
+    my @paths = @{ delete $options->{paths} || $self->paths };
+    my @ns    = @{ delete $options->{snippet_namespaces} || $self->snippet_namespaces };
+
+    $options->{$_} ||= $self->$_ for qw/ helper wrapper encoding /;
 
     Plift::Context->new(
+        %$options,
         template => $name,
-        helper   => $options->{helper}   || $self->helper,
-        wrapper  => $options->{wrapper}  || $self->wrapper,
-        encoding => $options->{encoding} || $self->encoding,
         handlers => [@{ $self->{handlers}}],
         load_template => sub {
             my ($ctx, $name) = @_;
@@ -369,7 +370,7 @@ __END__
 
 =head1 NAME
 
-Plift - Designer friendly, safe, extensible HTML template engine.
+Plift - Pure HTML templates with reusable custom elements.
 
 =head1 SYNOPSIS
 
