@@ -4,13 +4,20 @@ use Moo;
 use Carp;
 use Scalar::Util qw/ blessed /;
 use XML::LibXML::jQuery;
+use JSON 'to_json';
+use Encode 'encode';
+use namespace::clean;
+
 
 has 'helper', is => 'ro';
 has 'wrapper', is => 'ro';
 has 'template', is => 'ro', required => 1;
 has 'encoding', is => 'ro', default => 'UTF-8';
 has 'loop_var', is => 'ro', default => 'loop';
+has 'metadata_key', is => 'ro', default => 'meta';
 has 'handlers', is => 'ro', default => sub { [] };
+has 'active_handlers', is => 'rw';
+has 'inactive_handlers', is => 'rw';
 has 'internal_id_attribute', is => 'ro', default => 'data-plift-id';
 has '_load_template', is => 'ro', required => 1, init_arg => 'load_template';
 has '_load_snippet', is => 'ro', required => 1, init_arg => 'load_snippet';
@@ -19,8 +26,8 @@ has '_load_snippet', is => 'ro', required => 1, init_arg => 'load_snippet';
 has 'document', is => 'rw', init_arg => undef;
 has 'is_rendering', is => 'rw', init_arg => undef, default => 0;
 
-has '_data_stack',   is => 'ro', default => sub { [] };
-has '_directive_stack',   is => 'ro', default => sub { [] };
+has '_data_stack',   is => 'ro', init_arg => undef, default => sub { [] };
+has '_directive_stack',   is => 'ro', init_arg => undef, default => sub { [] };
 
 
 sub AUTOLOAD {
@@ -38,6 +45,25 @@ sub AUTOLOAD {
     $self->helper->$method(@_);
 }
 
+sub BUILD {
+    my $self = shift;
+
+    # coerse array indo hash
+    foreach my $attr (qw/ active_handlers inactive_handlers/) {
+
+        $self->$attr({ map { $_ => 1 } @{ $self->$attr } })
+            if $self->$attr;
+    }
+}
+
+sub metadata {
+    my $self = shift;
+    my $key = $self->metadata_key;
+    my $data = $self->data;
+    $data->{$key} = {} unless exists $data->{$key};
+    $data->{$key};
+}
+
 
 sub data {
     my $self = shift;
@@ -294,17 +320,32 @@ sub process_element {
         $self->_dispatch_handlers(@_, $element->_new_nodes([$_[1]]));
     };
 
-    # xpath
-    my $find_xpath = join ' | ', map { $_->{xpath} } @{ $self->handlers };
+    # get handlers
+    my @handlers = @{ $self->handlers };
+    @handlers = grep { exists $self->active_handlers->{$_->{name}} } @handlers
+        if defined $self->active_handlers;
+
+    @handlers = grep { !exists $self->inactive_handlers->{$_->{name}} } @handlers
+        if defined $self->inactive_handlers;
+
+    # build xpath
+    my $find_xpath = join ' | ', map { $_->{xpath} } @handlers;
     my $filter_xpath = $find_xpath;
     $filter_xpath =~ s{\.//}{./}g;
 
-    $element->xfilter($filter_xpath)->each($callback);
-    $element->xfind($find_xpath)->each($callback);
+    foreach my $node (
+        @{ $element->xfilter($filter_xpath)->{nodes} },
+        @{ $element->xfind($find_xpath)->{nodes} }
+    ) {
+
+        $self->_dispatch_handlers($node, $element->_new_nodes([$node]));
+    }
+
+    # printf STDERR "# afer filter: %s\n", $element->as_html;
 }
 
 sub _dispatch_handlers {
-    my ($self, $i, $node, $el) = @_;
+    my ($self, $node, $el) = @_;
     my $tagname = $node->localname;
 
     foreach my $handler (@{ $self->handlers }) {
@@ -339,7 +380,7 @@ sub _dispatch_handlers {
 sub render  {
     my ($self, $data) = @_;
 
-    $self->set($data)
+    @{ $self->_data_stack } = ( $data )
         if defined $data;
 
     # already rendering
@@ -389,6 +430,7 @@ sub _render_directives {
         my $action = $directives->[$i+1];
 
         my $target_element = $el->find($selector);
+        $target_element = $el->filter($selector) if $target_element->size == 0;
         next unless $target_element->size > 0;
 
         # Scalar
@@ -398,6 +440,12 @@ sub _render_directives {
 
             $target_element->remove unless defined $value;
 
+            # to_json
+            $value = to_json($value) if ref $value eq 'HASH';
+
+            # encode
+            $value = encode 'UTF-8', $value;
+
             if (defined $attribute && $attribute ne 'HTML') {
 
                 # TODO append prepend attribute
